/**
 * Created by Samuel Gratzl on 25.01.2016.
 */
import * as tslib_1 from "tslib";
import './style.scss';
import * as d3 from 'd3';
import { onDOMNodeRemoved, mixin } from 'phovea_core/src';
import { AVisInstance, assignVis } from 'phovea_core/src/vis';
import { rect } from 'phovea_core/src/geom';
import { toSelectOperation } from 'phovea_core/src/idtype';
var BarPlot = (function (_super) {
    tslib_1.__extends(BarPlot, _super);
    function BarPlot(data, parent, options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this) || this;
        _this.data = data;
        _this.options = {
            cssClass: '',
            width: 100,
            heighti: 10,
            min: 0,
            max: NaN,
            scale: [1, 1],
            rotate: 0
        };
        mixin(_this.options, options);
        _this.$node = _this.build(d3.select(parent));
        _this.$node.datum(_this);
        assignVis(_this.node, _this);
        return _this;
    }
    Object.defineProperty(BarPlot.prototype, "rawSize", {
        get: function () {
            return [this.options.width, this.data.dim[0] * this.options.heighti];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BarPlot.prototype, "node", {
        get: function () {
            return this.$node.node();
        },
        enumerable: true,
        configurable: true
    });
    BarPlot.prototype.build = function ($parent) {
        var _this = this;
        var o = this.options, size = this.size, data = this.data;
        var $svg = $parent.append('svg').attr({
            width: size[0],
            height: size[1],
            'class': 'phovea-barplot ' + o.cssClass
        });
        //using range bands with an ordinal scale for uniform distribution
        var xscale = this.xscale = d3.scale.linear().range([0, 100]);
        var yscale = this.yscale = d3.scale.linear().range([0, 100]);
        var onClick = function (d, i) {
            data.select(0, [i], toSelectOperation(d3.event));
        };
        var l = function (event, type, selected) {
            $svg.selectAll('rect').classed('phovea-select-' + type, false);
            if (selected.isNone) {
                return;
            }
            var dim0 = selected.dim(0);
            if (selected.isAll) {
                $svg.selectAll('rect').classed('phovea-select-' + type, true);
            }
            else {
                dim0.forEach(function (j) { return $svg.selectAll('rect:nth-child(' + (j + 1) + ')').classed('phovea-select-' + type, true); });
            }
        };
        data.on('select', l);
        onDOMNodeRemoved($svg.node(), function () { return data.off('select', l); });
        data.data().then(function (_data) {
            yscale.domain([0, data.length]);
            if (isNaN(o.min) || isNaN(o.max)) {
                var minmax = d3.extent(_data);
                if (isNaN(o.min)) {
                    o.min = minmax[0];
                }
                if (isNaN(o.max)) {
                    o.max = minmax[1];
                }
            }
            xscale.domain([o.min, o.max]);
            var $m = $svg.selectAll('rect').data(_data);
            $m.enter().append('rect')
                .on('click', onClick);
            $m.attr({
                y: function (d, i) { return yscale(i); },
                height: function (d) { return yscale(1); },
                width: xscale
            });
            _this.markReady();
            data.selections().then(function (selected) { return l(null, 'selected', selected); });
        });
        return $svg;
    };
    BarPlot.prototype.locateImpl = function (range) {
        var _this = this;
        var o = this.options;
        var exI = d3.extent(range.dim(0).iter().asList());
        return this.data.data(range).then(function (data) {
            var exV = d3.extent(data);
            return rect(_this.xscale(exV[0]) / 100.0 * o.width, exI[0] * o.heighti, _this.xscale(exV[1]) / 100.0 * o.width, (exI[1] + 1) * o.heighti);
        });
    };
    return BarPlot;
}(AVisInstance));
export { BarPlot };
export default BarPlot;
export function create(data, parent, options) {
    return new BarPlot(data, parent, options);
}
//# sourceMappingURL=barplot.js.map