/**
 * Created by Samuel Gratzl on 26.12.2016.
 */
import * as d3 from 'd3';
import { cell } from 'phovea_core/src/range';
import { onDOMNodeRemoved } from 'phovea_core/src';
import { toSelectOperation, defaultSelectionType } from 'phovea_core/src/idtype';
import { ESelectOption } from './IHeatMapRenderer';
var HeatMapDOMRenderer = (function () {
    function HeatMapDOMRenderer(selectAble) {
        if (selectAble === void 0) { selectAble = ESelectOption.CELL; }
        this.selectAble = selectAble;
    }
    HeatMapDOMRenderer.prototype.rescale = function ($node, dim, scale) {
        $node.select('svg').attr({
            width: dim[1] * scale[0],
            height: dim[0] * scale[1]
        });
        $node.select('g').attr('transform', 'scale(' + scale[0] + ',' + scale[1] + ')');
    };
    HeatMapDOMRenderer.prototype.recolor = function ($node, data, color, scale) {
        this.color = color;
        $node.select('svg').selectAll('rect').attr('fill', function (d) { return color(d); });
    };
    HeatMapDOMRenderer.prototype.redraw = function ($node, scale) {
        var _this = this;
        $node.select('svg').selectAll('rect').attr('fill', function (d) { return _this.color(d); });
    };
    HeatMapDOMRenderer.prototype.build = function (data, $parent, scale, c, onReady) {
        var dims = data.dim, that = this;
        var width = dims[1], height = dims[0];
        var $node = $parent.append('div').attr('class', 'phovea-heatmap');
        var $svg = $node.append('svg').attr({
            width: width * scale[0],
            height: height * scale[1]
        });
        var $g = $svg.append('g').attr('transform', 'scale(' + scale[0] + ',' + scale[1] + ')');
        this.color = c;
        data.data().then(function (arr) {
            var $rows = $g.selectAll('g').data(arr);
            $rows.enter().append('g').each(function (row, i) {
                var $cols = d3.select(this).selectAll('rect').data(row);
                var $colsEnter = $cols.enter().append('rect').attr({
                    width: 1,
                    height: 1,
                    x: function (d, j) { return j; },
                    y: i,
                    fill: function (d) { return c(d); }
                });
                if (that.selectAble !== ESelectOption.NONE) {
                    $colsEnter.on('click', function (d, j) {
                        data.selectProduct([cell(i, j)], toSelectOperation(d3.event));
                    });
                }
                $colsEnter.append('title').text(String);
            });
            onReady();
        });
        var l = function (event, type, selected) {
            $g.selectAll('rect').classed('phovea-select-' + type, false);
            if (selected.length === 0) {
                return;
            }
            selected.forEach(function (cell) {
                cell.product(function (indices) {
                    $g.select("g:nth-child(" + (indices[0] + 1) + ")").select("rect:nth-child(" + (indices[1] + 1) + ")").classed('phovea-select-' + type, true);
                }, data.dim);
            });
        };
        if (this.selectAble !== ESelectOption.NONE) {
            data.on('selectProduct', l);
            onDOMNodeRemoved($g.node(), function () {
                data.off('selectProduct', l);
            });
            data.productSelections().then(function (selected) {
                l(null, defaultSelectionType, selected);
            });
        }
        return $node;
    };
    return HeatMapDOMRenderer;
}());
export default HeatMapDOMRenderer;
//# sourceMappingURL=HeatMapDOMRenderer.js.map