/**
 * Created by Samuel Gratzl on 25.01.2016.
 */
import * as tslib_1 from "tslib";
import './style.scss';
import { select, extent, format } from 'd3';
import { mixin } from 'phovea_core/src';
import { AVisInstance, assignVis } from 'phovea_core/src/vis';
import { selectionUtil } from 'phovea_d3/src/d3util';
import { rect } from 'phovea_core/src/geom';
var List = (function (_super) {
    tslib_1.__extends(List, _super);
    function List(data, parent, options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this) || this;
        _this.data = data;
        _this.options = {
            format: null,
            scale: [1, 1],
            rotate: 0,
            width: 200,
            rowHeight: 20
        };
        mixin(_this.options, options);
        _this.$node = _this.build(select(parent));
        _this.$node.datum(_this);
        assignVis(_this.node, _this);
        return _this;
    }
    Object.defineProperty(List.prototype, "rawSize", {
        get: function () {
            return [this.options.width, this.data.length * this.options.rowHeight];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(List.prototype, "node", {
        get: function () {
            return this.$node.node();
        },
        enumerable: true,
        configurable: true
    });
    List.prototype.locateImpl = function (range) {
        var w = this.node.clientWidth;
        var $node = select(this.node);
        var a, b;
        if (range.isAll || range.isNone) {
            a = this.node.firstElementChild;
            b = this.node.lastElementChild;
        }
        else {
            var ex = extent(range.dim(0).iter().asList());
            a = this.node.childNodes.item(ex[0]);
            b = this.node.childNodes.item(ex[1]);
        }
        return Promise.resolve(rect(0, a.offsetTop, w, b.offsetTop + b.clientHeight - a.offsetTop));
    };
    List.prototype.transform = function (scale, rotate) {
        if (rotate === void 0) { rotate = 0; }
        var bak = {
            scale: this.options.scale || [1, 1],
            rotate: this.options.rotate || 0
        };
        if (arguments.length === 0) {
            return bak;
        }
        this.$node.style('transform', 'rotate(' + rotate + 'deg)');
        this.$node.style('width', scale[0] * this.options.width + "px");
        this.$node.style('height', scale[1] * this.data.length * this.options.rowHeight + "px");
        var act = { scale: scale, rotate: rotate };
        this.fire('transform', act, bak);
        this.options.scale = scale;
        this.options.rotate = rotate;
        return act;
    };
    List.prototype.build = function ($parent) {
        var _this = this;
        var scale = this.options.scale;
        var $list = $parent.append('div').attr('class', 'phovea-list');
        $list.style('width', scale[0] * this.options.width + "px");
        $list.style('height', scale[1] * this.data.length * this.options.rowHeight + "px");
        var onClick = selectionUtil(this.data, $list, 'div');
        this.data.data().then(function (arr) {
            var $rows = $list.selectAll('div').data(arr);
            $rows.enter().append('div').on('click', onClick);
            var formatter = _this.options.format ? format(_this.options.format) : String;
            $rows.text(formatter);
            $rows.exit().remove();
            _this.markReady();
        });
        return $list;
    };
    return List;
}(AVisInstance));
export { List };
export default List;
export function create(data, parent, options) {
    return new List(data, parent, options);
}
//# sourceMappingURL=list.js.map