/**
 * Created by Marc Streit on 06.08.2014.
 */
import * as tslib_1 from "tslib";
import './style.scss';
import * as d3 from 'd3';
import { AVisInstance, assignVis } from 'phovea_core/src/vis';
import { mixin } from 'phovea_core/src';
import bindTooltip from 'phovea_d3/src/tooltip';
var ScatterPlot = (function (_super) {
    tslib_1.__extends(ScatterPlot, _super);
    function ScatterPlot(data, parent, options) {
        if (options === void 0) { options = {}; }
        var _this = _super.call(this) || this;
        _this.data = data;
        _this.options = {
            scale: [1, 1],
            rotate: 0
        };
        mixin(_this.options, options);
        _this.$node = _this.build(d3.select(parent));
        _this.$node.datum(_this);
        assignVis(_this.$node.node(), _this);
        return _this;
    }
    Object.defineProperty(ScatterPlot.prototype, "rawSize", {
        get: function () {
            return [300, 300];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ScatterPlot.prototype, "node", {
        get: function () {
            return this.$node.node();
        },
        enumerable: true,
        configurable: true
    });
    ScatterPlot.prototype.build = function ($parent) {
        var size = this.size, data = this.data;
        var width = size[0], height = size[1];
        var xcol = 0;
        var ycol = 1;
        var svg = $parent.append('svg').attr({
            width: width,
            height: height,
            'class': 'phovea-scatterplot'
        });
        var that = this;
        // bind data to chart
        Promise.all([data.data(), data.rows()]).then(function (promise) {
            var arr = promise[0];
            var rowNames = promise[1];
            // create scales
            var x = d3.scale.linear().domain([0, d3.max(arr, function (d) { return d[xcol]; })]).range([0, width]), y = d3.scale.linear().domain([0, d3.max(arr, function (d) { return d[ycol]; })]).range([height, 0]);
            svg.selectAll('circle')
                .data(arr)
                .enter().append('circle')
                .attr('cx', function (d) { return x(d[xcol]); })
                .attr('cy', function (d) { return y(d[ycol]); })
                .attr('r', 2)
                .call(bindTooltip(function (d, i) { return rowNames[i]; }));
        });
        function update() {
            that.data.data().then(function (arr) {
                // create scales
                var x = d3.scale.linear().domain([0, d3.max(arr, function (d) { return d[xcol]; })]).range([0, width]), y = d3.scale.linear().domain([0, d3.max(arr, function (d) { return d[ycol]; })]).range([height, 0]);
                svg.selectAll('circle')
                    .transition()
                    .ease('linear')
                    .duration(1000)
                    .attr('cx', function (d) { return x(d[xcol]); })
                    .attr('cy', function (d) { return y(d[ycol]); });
            });
        }
        var $xaxis = $parent.append('select')
            .on('change', function () {
            xcol = this.selectedIndex;
            update();
        });
        var $yaxis = $parent.append('select')
            .on('change', function () {
            ycol = this.selectedIndex;
            update();
        });
        data.cols().then(function (cols) {
            var $x = $xaxis.selectAll('option').data(cols);
            $x.enter().append('option');
            $x.attr('value', function (d, i) { return i; })
                .text(String)
                .each(function (d, i) {
                if (i === xcol) {
                    d3.select(this).attr('selected', 'selected');
                }
            });
            $x.exit().remove();
            var $y = $yaxis.selectAll('option').data(cols);
            $y.enter().append('option');
            $y.attr('value', function (d, i) { return i; })
                .text(String)
                .each(function (d, i) {
                if (i === ycol) {
                    d3.select(this).attr('selected', 'selected');
                }
            });
            $y.exit().remove();
        });
        return svg;
    };
    return ScatterPlot;
}(AVisInstance));
export { ScatterPlot };
export function create(data, parent, options) {
    return new ScatterPlot(data, parent, options);
}
//# sourceMappingURL=scatterplot.js.map